from collections import defaultdict
import itertools
from pathlib import Path
from typing import Dict, Optional, Sequence, Tuple
import yaml

from opa_tools.utils import process_dct


def flatten(gsvrequest, oparequest):
    return {
        **process_dct(gsvrequest, prefix="gsv_"),
        **process_dct(oparequest, prefix="opa_"),
    }


def unflatten(flatrequest):
    return (
        {
            key.lstrip("gsv_"): value
            for key, value in flatrequest.items()
            if key.startswith("gsv_")
        },
        {
            key.lstrip("opa_"): value
            for key, value in flatrequest.items()
            if key.startswith("opa_")
        },
    )


def product(infile: Path, outpath: Optional[Path] = None):
    with open(infile, "r") as rf:
        req_items = list(yaml.safe_load(rf).items())
    requests = []
    req_keys, req_values = zip(*req_items)
    for value_comb in itertools.product(*req_values):
        requests.append(dict(zip(req_keys, value_comb)))
    return requests


def wrap(
    requests: Sequence[Tuple[Dict, Dict]], /, identifier, outfile: Optional[Path] = None
):
    req_dcts = defaultdict(set)
    for gsvrequest, oparequest in requests:
        request = flatten(gsvrequest, oparequest)
        for key, value in request.items():
            # Making hashable in a cheap way by turning into string
            req_dcts[key].add(value)
    req_dcts = {key: list(values) for key, values in req_dcts.items()}
    if outfile:
        with open(outfile, "w") as wf:
            yaml.dump(req_dcts, wf)
    return req_dcts
